@@ -3,6 +3,7 @@ import re
 import sys
 import typing
 from dataclasses import dataclass
+from typing import Annotated, Any
 
 import pytest
 
@@ -21,7 +22,7 @@ from ovld import (
 )
 from ovld.dependent import Dependent, Equals, Regexp, StartsWith
 from ovld.types import UnionTypes
-from ovld.utils import MISSING, UsageError
+from ovld.utils import MISSING, ResolutionError, UsageError
 
 from .test_typemap import Animal, Bird, Mammal
 
@@ -808,6 +809,20 @@ def test_priority():
     assert f([1, "x"]) == ["TOP", [["TOP", 2], ["TOP", "BOTTOM"]]]
 
 
+def test_priority_tuple():
+    f = Ovld()
+
+    @f.register(priority=(1, 2))
+    def f(x: object):
+        return ["TOP", call_next(x)]
+
+    @f.register(priority=(1, 1))
+    def f(x: object):
+        return "BOTTOM"
+
+    assert f(123) == ["TOP", "BOTTOM"]
+
+
 def test_resolve_for_values():
     f = Ovld()
 
@@ -1145,18 +1160,6 @@ def test_replacement():
     assert f(5) == 2
 
 
-def test_disallow_replacement():
-    @ovld(allow_replacement=False)
-    def f(x: int):
-        pass
-
-    with pytest.raises(TypeError):
-
-        @f.register
-        def f2(x: int):
-            pass
-
-
 def test_unregister():
     @ovld
     def f(xs: list):
@@ -1277,6 +1280,81 @@ def test_generic_type_argument():
     assert f(dict[str, int]) == "dict"
 
 
+def test_annotated_type_argument():
+    @ovld
+    def f(t: type[Annotated[str, "hello"]]):
+        return "hello"
+
+    @ovld
+    def f(t: type[Annotated[str, "world"]]):
+        return "world"
+
+    @ovld
+    def f(t: object):
+        return False
+
+    assert f(Annotated[str, "hello"]) == "hello"
+    assert f(Annotated[str, "world"]) == "world"
+    assert f(Annotated[str, "hello", "X"]) == "hello"
+    assert f(Annotated[str, 1, 2, 3, "world"]) == "world"
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 12), reason="Python 3.11 is more strict on Annotated"
+)
+def test_annotated_type_argument_any():
+    @ovld
+    def f(t: type[Annotated[Any, "hello"]]):
+        return "hello"
+
+    @ovld
+    def f(t: object):
+        return False
+
+    assert f(Annotated[str, "hello"]) == "hello"
+    assert f(Annotated[123, "hello", "X"]) == "hello"
+    assert f(Annotated[str, 1, 2, 3, "hello"]) == "hello"
+
+
+@dataclass(frozen=True)
+class Anno:
+    name: str
+    annotation_priority: int = 0
+
+
+def test_annotated_multiple_ann():
+    hello = Anno("hello", 1)
+    world = Anno("hello", 2)
+    foo = Anno("foo", 1)
+    bar = Anno("bar", 1)
+
+    @ovld
+    def f(t: type[Annotated[str, hello]]):
+        return "hello"
+
+    @ovld
+    def f(t: type[Annotated[str, world]]):
+        return "world"
+
+    @ovld
+    def f(t: type[Annotated[str, foo]]):
+        return "foo"
+
+    @ovld
+    def f(t: type[Annotated[object, bar]]):
+        return "bar"
+
+    @ovld
+    def f(t: object):
+        return False
+
+    assert f(Annotated[str, hello, world]) == "world"
+    assert f(Annotated[str, hello, bar]) == "hello"
+    with pytest.raises(ResolutionError):
+        assert f(Annotated[str, hello, foo]) == "world"
+    assert f("ah") is False
+
+
 def test_any():
     @ovld
     def f(x: int):
@@ -1743,12 +1821,12 @@ def test_doc(file_regression):
         return None
 
     doc = f"{mushroom.__name__}{inspect.signature(mushroom)}\n\n"
-    doc += mushroom.__doc__
+    doc += inspect.getdoc(mushroom)
 
     file_regression.check(doc)
 
-    assert mushroom.__doc__ == mushroom.__ovld__.__doc__
-    assert mushroom.__signature__ == mushroom.__ovld__.__signature__
+    assert inspect.getdoc(mushroom) == inspect.getdoc(mushroom.__ovld__)
+    assert inspect.signature(mushroom) == inspect.signature(mushroom.__ovld__)
 
 
 def test_doc2(file_regression):
@@ -1765,7 +1843,7 @@ def test_doc2(file_regression):
     def mushroom(x: str, y: object, *, beauty, bigness):
         return None
 
-    doc = mushroom.__doc__
+    doc = inspect.getdoc(mushroom)
     doc = f"{mushroom.__name__}{inspect.signature(mushroom)}\n\n" + doc
 
     file_regression.check(doc)
@@ -1787,7 +1865,7 @@ def test_method_doc(file_regression):
             return None
 
     mushroom = Mushroom()
-    doc = mushroom.rise.__doc__
+    doc = inspect.getdoc(mushroom.rise)
     doc = f"{mushroom.rise.__name__}{inspect.signature(mushroom.rise)}\n\n" + doc
 
     file_regression.check(doc)
